@@ -21,9 +21,6 @@
  ****************************************************************
  */
 
-#include "rcs.h"
-RCS_ID("$Id: teln.c,v 1.6 1994/05/31 12:32:15 mlschroe Exp $ FAU")
-
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <fcntl.h>
@@ -233,7 +230,7 @@ int *lenp;
 	    }
 	  fore->w_telbufl--;
 	}
-      if ((c >= 0x20 && c <= 0x7e) || (c >= 0xa0 && c <= 0xff))
+      if ((c >= 0x20 && c <= 0x7e) || c >= 0xa0)
 	{
 	  if (echo)
 	    WriteString(fore, (char *)&c, 1);
